/*

Copyright (C) 2013 Andrej Lojdl

This file is part of Octave.

Octave is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

Octave is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Octave; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include"oct.h"
#include"octave.h"
#include"parse.h"

#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include "txt-latex.h"


latex_render::latex_render (void)
: bbox (1, 4, 0.0),red (0),green (0),blue (0) 
{
}

latex_render::~latex_render (void)
{
}

void 
latex_render::adapter (const std::string& txt)
{
  std::ofstream file;

  file.open ("default.tex");
  /* Adding LaTeX configuration lines to this file */
  file << "\\documentclass[fleqn]{article} \n\\usepackage{amsmath} \n\\usepackage{color} \n\\pagestyle{empty}	\n\\begin{document} \n\t\\normalsize \n\t\t\\begin{displaymath}\n\t\t\t";
  file << txt;
  file << "\n\t\t\\end{displaymath} \n\\end{document}";
  file.close();	
}

uint8NDArray 
latex_render::render (void)
{
  int cmd,width,height,max_color;
  std::string comment;
  uint8NDArray data;
  std::ifstream file;
  char magic_number[2],skip;

  // Checking if command processor is available //
  if(system( NULL ))
    {
      /* Probably we want to delete files after they are no longer needed - checking if user have all programs will be done soewhere else */

      // .tex -> .dvi
      cmd = system ("latex default.tex");

      // .dvi -> .eps
      cmd = system ("dvips default.dvi -E -o default.eps");

      // .eps -> .png 
      cmd = system ("gs -dEPSCrop -dSAFER -dBATCH -dNOPAUSE -r300 -sDEVICE=ppm -sOutputFile=default.ppm \"default.eps\"");

      // removing temporary files 
      cmd = remove ("default.tex");
      cmd = remove ("default.dvi");
      cmd = remove ("default.log");
      cmd = remove ("default.aux");
      cmd = remove ("default.eps");

      // read data from PPM file to Array for OpenGL rendering
      file.open("default.ppm");
      file>>magic_number>>skip;
      std::getline(file, comment);
      file>>width>>height;
      data = uint8NDArray (dim_vector (4, width, height), static_cast<uint8_t> (0));

      // reading the data, row by row 
      for(int i=height-1; i>=0; i--)
        {
          for(int j=0; j<width; j++)
            {
              int r,g,b;
              
              file >> r >> g >> b;
              data(0,j,i) = r; //red
              data(1,j,i) = g; //green
              data(2,j,i) = b; //blue
              data(3,j,i) = 255;   //alpha -> transparent
            }
        }
      file.close();
    }
  else
    exit (EXIT_FAILURE);
    
  
  
  return data; 
}

Matrix 
latex_render::get_bbox (void)
{
  Matrix bounding_box (1, 4, 0.0);
  std::ifstream file;
  std::string skip;
  int width, height;
  
  file.open("default.ppm");

  std::getline(file, skip);
  std::getline(file, skip);

  
  file>>width>>height;
  
  bounding_box(2) = width;
  bounding_box(3) = height;

  file.close();
  
  remove("default.ppm");
  
  return bounding_box;
}

void
latex_render::set_font (const std::string& name, const std::string& weight,
                        const std::string& angle, double size)
{}

void
latex_render::set_color (Matrix c) {}

void
latex_render::text_to_pixels (const std::string& txt,
                              uint8NDArray& pixels, Matrix& bbox,
                              int halign, int valign, double rotation)
{
  adapter (txt);
  pixels = render();
  bbox = get_bbox();	
}

Matrix
latex_render::get_extent (text_element *elt, double rotation)
{
  Matrix extent (1, 2, 0.0);

  return extent;
}


Matrix 
latex_render::get_extent (const std::string& txt, double rotation)
{
  text_element *elt = text_parser_none().parse(txt);
  Matrix extent = get_extent (elt, rotation);
  delete elt;

  return extent;
}
